#ifndef _util_c_
#define _util_c_
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>

void
fatalError (const char *message)
{
  fprintf (stderr, "FATAL ERROR: %s\n", message);
  exit (EXIT_FAILURE);
}

void *
pmalloc (size_t nbytes)
{
  void *p = malloc (nbytes);
  if (!p)
    {
      fatalError ("memory allocation Failed in pmalloc");
    }
  return p;
}

void *
pcalloc (size_t nmem, size_t memsize)
{
  void *p = calloc (nmem, memsize);
  if (!p)
    {
      fatalError ("memory allocation Failed in pcalloc");
    }
  return p;
}

void *
prealloc (void *old_ptr, size_t nbytes)
{
  void *p = realloc (old_ptr, nbytes);
  if (!p)
    {
      fatalError ("memory allocation Failed in prealloc");
    }
  return p;
}

FILE *
openFile (const char *fname)
{
  FILE *fp = fopen (fname, "r");
  if (!fp)
    {
      fprintf (stderr, "Could not open the file %s for reading!\n", fname);
      fatalError ("Check to see if file exists");
    }
  return fp;
}

FILE *
createFile (const char *fname)
{
  FILE *fp = fopen (fname, "w");
  if (!fp)
    {
      fprintf (stderr, "Could not create the file %s !\n", fname);
      fatalError ("Check directory permissions or for too many open files");
    }
  return fp;
}

FILE *
openAppendFile (const char *fname)
{
  FILE *fp = fopen (fname, "a");
  if (!fp)
    {
      fprintf (stderr, "Could not open the file %s for appending!\n", fname);
      fatalError ("Check to see if file exists");
    }
  return fp;
}

double *
dalloc (size_t n)
{
  return (double *) pmalloc (sizeof (double) * n);
}

double **
dmatrix (size_t n1, size_t n2)
{
  int i;
  double **m = (double **) pmalloc (sizeof (double *) * n1);
  for (i = 0; i < n1; ++i)
    {
      m[i] = (double *) pmalloc (sizeof (double) * n2);
    }
  return m;
}

double ***
dtensor (size_t n1, size_t n2, size_t n3)
{
  int i, j;
  double ***t = (double ***) pmalloc (sizeof (double **) * n1);
  for (i = 0; i < n1; ++i)
    {
      t[i] = (double **) pmalloc (sizeof (double *) * n2);
      for (j = 0; j < n2; ++j)
	{
	  t[i][j] = (double *) pmalloc (sizeof (double) * n3);
	}
    }
  return t;
}

void
free_dmatrix (double **m, size_t n1)
{
  register int i = n1;
  for (; i--;)
    free (m[i]);
  free (m);
}

void
free_dtensor (double ***t, size_t n1, size_t n2)
{
  register int i, j;
  for (i = n1; i--;)
    {
      for (j = n2; j--;)
	{
	  free (t[i][j]);
	}
      free (t[i]);
    }
  free (t);
}

int *
ialloc (size_t n)
{
  return (int *) pmalloc (sizeof (int) * n);
}

int **
imatrix (size_t n1, size_t n2)
{
  int i;
  int **m = (int **) pmalloc (sizeof (int *) * n1);
  for (i = 0; i < n1; ++i)
    {
      m[i] = (int *) pmalloc (sizeof (int) * n2);
    }
  return m;
}

int ***
itensor (size_t n1, size_t n2, size_t n3)
{
  int i, j;
  int ***t = (int ***) pmalloc (sizeof (int **) * n1);
  for (i = 0; i < n1; ++i)
    {
      t[i] = (int **) pmalloc (sizeof (int *) * n2);
      for (j = 0; j < n2; ++j)
	{
	  t[i][j] = (int *) pmalloc (sizeof (int) * n3);
	}
    }
  return t;
}

void
free_imatrix (int **m, size_t n1)
{
  register int i = n1;
  for (; i--;)
    free (m[i]);
  free (m);
}

void
free_itensor (int ***t, size_t n1, size_t n2)
{
  register int i, j;
  for (i = n1; i--;)
    {
      for (j = n2; j--;)
	{
	  free (t[i][j]);
	}
      free (t[i]);
    }
  free (t);
}

/** extra math functions **/
int
feqZero (const double x)
{
  if (fabs (x) <= DBL_EPSILON)
    return 1;
  return 0;
}

double
distSqr (const double *a, const double *b)
{
  register double x, y, z;
  x = a[0] - b[0];
  y = a[1] - b[1];
  z = a[2] - b[2];
  return (x * x + y * y + z * z);
}

double
normSqr (const double *a)
{
  return (a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
}

int
fcmp (const double x, const double y)
{
  register double z = x - y;
  if (fabs (z) <= DBL_EPSILON)
    return 0;
  if (z > 0.)
    return 1;
  return -1;
}

#endif
